@@ -42,6 +42,7 @@ def make_rows(num_columns, seq):
 	# of rows
 	return zip(*result)
 
+
 def bisect(seq, func=bool):
 	"""
 	Split a sequence into two sequences:  the first is elements that
@@ -56,7 +57,7 @@ def bisect(seq, func=bool):
 	>>> list(even)
 	[0, 2, 4]
 
-	>>> zeros, other = bisect(reversed(range(5)))
+	>>> other, zeros = bisect(reversed(range(5)))
 	>>> list(zeros)
 	[0]
 	>>> list(other)
@@ -66,6 +67,7 @@ def bisect(seq, func=bool):
 	queues = GroupbySaved(seq, func)
 	return queues.get_first_n_queues(2)
 
+
 class GroupbySaved(object):
 	"""
 	Split a sequence into n sequences where n is determined by the
@@ -99,10 +101,11 @@ class GroupbySaved(object):
 	>>> next(ones)
 	4
 	"""
-	def __init__(self, sequence, func = lambda x: x):
+
+	def __init__(self, sequence, func=lambda x: x):
 		self.sequence = iter(sequence)
 		self.func = func
-		self.queues = dict()
+		self.queues = collections.OrderedDict()
 
 	def __getitem__(self, key):
 		try:
@@ -120,7 +123,7 @@ class GroupbySaved(object):
 	def __find_queue__(self, key):
 		"search for the queue indexed by key"
 		try:
-			while not key in self.queues:
+			while key not in self.queues:
 				self.__fetch__()
 			return self.queues[key]
 		except StopIteration:
@@ -142,6 +145,7 @@ class GroupbySaved(object):
 		values.extend(iter([]) for n in range(missing))
 		return values
 
+
 class FetchingQueue(queue.Queue):
 	"""
 	A FIFO Queue that is supplied with a function to inject more into
@@ -157,6 +161,7 @@ class FetchingQueue(queue.Queue):
 	tuple(q) just copies the elements in the list (of which there are
 	none).
 	"""
+
 	def __init__(self, fetcher):
 		if six.PY3:
 			super(FetchingQueue, self).__init__()
@@ -172,11 +177,15 @@ class FetchingQueue(queue.Queue):
 
 	def __iter__(self):
 		while True:
-			yield next(self)
+			try:
+				yield next(self)
+			except StopIteration:
+				return
 
 	def enqueue(self, item):
 		self.put_nowait(item)
 
+
 class Count(object):
 	"""
 	A stop object that will count how many times it's been called and return
@@ -208,6 +217,7 @@ class Count(object):
 
 	If all you need is the count of items, consider :class:`Counter` instead.
 	"""
+
 	def __init__(self, limit):
 		self.count = 0
 		self.limit = limit if limit is not None else float('Inf')
@@ -245,6 +255,7 @@ class islice(object):
 	>>> print(islice(3, 10, 2))
 	every 2nd item from 3 to 9
 	"""
+
 	def __init__(self, *sliceArgs):
 		self.sliceArgs = sliceArgs
 
@@ -259,7 +270,10 @@ class islice(object):
 		return result
 
 	def _formatArgs(self):
-		slice_range = lambda a_b: '%d to %d' % (a_b[0], a_b[1] - 1)
+
+		def slice_range(a_b):
+			return '%d to %d' % (a_b[0], a_b[1] - 1)
+
 		if len(self.sliceArgs) == 1:
 			result = 'at most %d' % self.sliceArgs
 		if len(self.sliceArgs) == 2:
@@ -270,6 +284,7 @@ class islice(object):
 			result = 'every %(ord)s item from %(range)s' % locals()
 		return result
 
+
 class LessThanNBlanks(object):
 	"""
 	An object that when called will return True until n false elements
@@ -278,7 +293,8 @@ class LessThanNBlanks(object):
 	Can be used with filter or itertools.ifilter, for example:
 
 	>>> import itertools
-	>>> sampleData = ['string 1', 'string 2', '', 'string 3', '', 'string 4', '', '', 'string 5']
+	>>> sampleData = ['string 1', 'string 2', '', 'string 3', '',
+	...     'string 4', '', '', 'string 5']
 	>>> first = itertools.takewhile(LessThanNBlanks(2), sampleData)
 	>>> tuple(first)
 	('string 1', 'string 2', '', 'string 3')
@@ -286,6 +302,7 @@ class LessThanNBlanks(object):
 	>>> tuple(first)
 	('string 1', 'string 2', '', 'string 3', '', 'string 4')
 	"""
+
 	def __init__(self, nBlanks):
 		self.limit = nBlanks
 		self.count = 0
@@ -296,6 +313,7 @@ class LessThanNBlanks(object):
 			raise ValueError("Should not call this object anymore.")
 		return self.count < self.limit
 
+
 class LessThanNConsecutiveBlanks(object):
 	"""
 	An object that when called will return True until n consecutive
@@ -304,7 +322,8 @@ class LessThanNConsecutiveBlanks(object):
 	Can be used with filter or itertools.ifilter, for example:
 
 	>>> import itertools
-	>>> sampleData = ['string 1', 'string 2', '', 'string 3', '', 'string 4', '', '', 'string 5']
+	>>> sampleData = ['string 1', 'string 2', '', 'string 3', '', 'string 4',
+	...     '', '', 'string 5']
 	>>> first = itertools.takewhile(LessThanNConsecutiveBlanks(2), sampleData)
 	>>> tuple(first)
 	('string 1', 'string 2', '', 'string 3', '', 'string 4', '')
@@ -324,6 +343,7 @@ class LessThanNConsecutiveBlanks(object):
 			raise ValueError("Should not call this object anymore.")
 		return self.count < self.limit
 
+
 class splitter(object):
 	"""
 	object that will split a string with the given arguments for each call.
@@ -332,7 +352,8 @@ class splitter(object):
 	>>> list(s('hello, world, this is your, master calling'))
 	['hello', ' world', ' this is your', ' master calling']
 	"""
-	def __init__(self, sep = None):
+
+	def __init__(self, sep=None):
 		self.sep = sep
 
 	def __call__(self, s):
@@ -346,6 +367,7 @@ class splitter(object):
 				yield s[lastIndex:]
 				break
 
+
 def grouper_nofill_str(n, iterable):
 	"""
 	Take a sequence and break it up into chunks of the specified size.
@@ -401,6 +423,7 @@ class Counter(object):
 	>>> items.count
 	20
 	"""
+
 	def __init__(self, i):
 		self.count = 0
 		self._orig_iter = iter(i)
@@ -419,8 +442,12 @@ class Counter(object):
 		return self.count
 
 # todo, factor out caching capability
+
+
 class iterable_test(dict):
 	"""
+	*Deprecated*
+
 	Test objects for iterability, caching the result by type
 
 	>>> test = iterable_test()
@@ -429,10 +456,12 @@ class iterable_test(dict):
 	>>> test[[]]
 	True
 	"""
-	def __init__(self, ignore_classes=six.string_types+(six.binary_type,)):
+
+	def __init__(self, ignore_classes=six.string_types + (six.binary_type,)):
 		"""ignore_classes must include str, because if a string
 		is iterable, so is a single character, and the routine runs
 		into an infinite recursion"""
+		warnings.warn("Slated for removal", DeprecationWarning)
 		assert set(six.string_types) <= set(ignore_classes), (
 			'str must be in ignore_classes')
 		self.ignore_classes = ignore_classes
@@ -451,6 +480,7 @@ class iterable_test(dict):
 		self[type(candidate)] = result
 		return result
 
+
 def iflatten(subject, test=None):
 	if test is None:
 		test = iterable_test()
@@ -461,8 +491,11 @@ def iflatten(subject, test=None):
 			for subelem in iflatten(elem, test):
 				yield subelem
 
+
 def flatten(subject, test=None):
 	"""
+	*Deprecated*: Use more_itertools.collapse instead.
+
 	Flatten an iterable with possible nested iterables.
 
 	Adapted from
@@ -481,7 +514,9 @@ def flatten(subject, test=None):
 	>>> flatten([b'ab', b'c'])
 	[b'ab', b'c']
 	"""
-	return list(iflatten(subject, test))
+	warnings.warn("Use more_itertools.collapse instead", DeprecationWarning)
+	return list(more.collapse(subject, base_type=(bytes,)))
+
 
 def empty():
 	"""
@@ -489,6 +524,7 @@ def empty():
 	"""
 	return iter(tuple())
 
+
 def is_empty(iterable):
 	"""
 	Return whether the iterable is empty or not. Consumes at most one item
@@ -505,6 +541,7 @@ def is_empty(iterable):
 		return True
 	return False
 
+
 class Reusable(object):
 	"""
 	An iterator that may be reset and reused.
@@ -527,7 +564,8 @@ class Reusable(object):
 		self.__saved = iterable
 		self.reset()
 
-	def __iter__(self): return self
+	def __iter__(self):
+		return self
 
 	def reset(self):
 		"""
@@ -547,6 +585,7 @@ class Reusable(object):
 			raise
 	next = __next__
 
+
 def every_other(iterable):
 	"""
 	Yield every other item from the iterable
@@ -556,8 +595,12 @@ def every_other(iterable):
 	"""
 	items = iter(iterable)
 	while True:
-		yield next(items)
-		next(items)
+		try:
+			yield next(items)
+			next(items)
+		except StopIteration:
+			return
+
 
 def remove_duplicates(iterable, key=None):
 	"""
@@ -577,6 +620,7 @@ def remove_duplicates(iterable, key=None):
 			itertools.groupby(iterable, key)
 		)))
 
+
 def skip_first(iterable):
 	"""
 	Skip the first element of an iterable
@@ -586,6 +630,7 @@ def skip_first(iterable):
 	"""
 	return itertools.islice(iterable, 1, None)
 
+
 def peek(iterable):
 	"""
 	Get the next value from an iterable, but also return an iterable
@@ -602,6 +647,7 @@ def peek(iterable):
 	peeker, original = itertools.tee(iterable)
 	return next(peeker), original
 
+
 class Peekable(object):
 	"""
 	Wrapper for a traditional iterable to give it a peek attribute.
@@ -672,7 +718,7 @@ def takewhile_peek(predicate, iterable):
 		yield next(iterable)
 
 
-def first(iterable):
+def first(iterable, *args):
 	"""
 	Return the first item from the iterable.
 
@@ -684,9 +730,21 @@ def first(iterable):
 	>>> first(iter)
 	0
 
+	Raises StopIteration if no value is present.
+
+	>>> first([])
+	Traceback (most recent call last):
+	...
+	StopIteration
+
+	Pass a default to be used when iterable is empty.
+
+	>>> first([], None)
+
 	"""
 	iterable = iter(iterable)
-	return next(iterable)
+	return next(iterable, *args)
+
 
 def last(iterable):
 	"""
@@ -706,6 +764,7 @@ def last(iterable):
 	except NameError:
 		raise ValueError("Iterable contains no items")
 
+
 def one(item):
 	"""
 	Return the first element from the iterable, but raise an exception
@@ -735,6 +794,7 @@ def one(item):
 	result, = item
 	return result
 
+
 def nwise(iter, n):
 	"""
 	Like pairwise, except returns n-tuples of adjacent items.
@@ -746,6 +806,7 @@ def nwise(iter, n):
 		next(iterset[-1], None)
 	return six.moves.zip(*iterset)
 
+
 def window(iter, pre_size=1, post_size=1):
 	"""
 	Given an iterable, return a new iterable which yields triples of
@@ -773,6 +834,7 @@ def window(iter, pre_size=1, post_size=1):
 	next(post_iter, None)
 	return six.moves.zip(pre_iter, iter, post_iter)
 
+
 class IterSaver(object):
 	def __init__(self, n, iterable):
 		self.n = n
@@ -785,6 +847,7 @@ class IterSaver(object):
 		return self.buffer.popleft()
 	next = __next__
 
+
 def partition_items(count, bin_size):
 	"""
 	Given the total number of items, determine the number of items that
@@ -808,6 +871,7 @@ def partition_items(count, bin_size):
 		bins[i % num_bins] += 1
 	return bins
 
+
 def balanced_rows(n, iterable, fillvalue=None):
 	"""
 	Like grouper, but balance the rows to minimize fill per row.
@@ -821,6 +885,7 @@ def balanced_rows(n, iterable, fillvalue=None):
 			row = itertools.chain(row, [fillvalue])
 		yield tuple(row)
 
+
 def reverse_lists(lists):
 	"""
 	>>> reverse_lists([[1,2,3], [4,5,6]])
@@ -829,6 +894,7 @@ def reverse_lists(lists):
 
 	return list(map(list, map(reversed, lists)))
 
+
 def always_iterable(item):
 	"""
 	Given an object, always return an iterable. If the item is not
@@ -836,13 +902,13 @@ def always_iterable(item):
 	None, an empty iterable is returned.
 
 	>>> always_iterable([1,2,3])
-	[1, 2, 3]
+	<list_iterator...>
 	>>> always_iterable('foo')
-	('foo',)
+	<tuple_iterator...>
 	>>> always_iterable(None)
-	()
+	<tuple_iterator...>
 	>>> always_iterable(range(10))
-	range(0, 10)
+	<range_iterator...>
 	>>> def _test_func(): yield "I'm iterable"
 	>>> print(next(always_iterable(_test_func())))
 	I'm iterable
@@ -853,14 +919,12 @@ def always_iterable(item):
 	>>> always_iterable(dict(a=1))
 	({'a': 1},)
 	"""
-	if item is None:
-		item = ()
-	singleton = (
-		isinstance(item, six.string_types)
-		or isinstance(item, collections.Mapping)
-		or not hasattr(item, '__iter__')
+	return (
+		(item,)
+		if isinstance(item, collections.Mapping)
+		else more.always_iterable(item)
 	)
-	return (item,) if singleton else item
+
 
 def suppress_exceptions(callables, *exceptions):
 	"""
@@ -885,6 +949,7 @@ def suppress_exceptions(callables, *exceptions):
 		except exceptions:
 			pass
 
+
 def apply(func, iterable):
 	"""
 	Like 'map', invoking func on each item in the iterable,
@@ -906,16 +971,26 @@ def apply(func, iterable):
 
 
 def list_or_single(iterable):
-    """
-    Given an iterable, return the items as a list. If the iterable contains
-    exactly one item, return that item. Correlary function to always_iterable.
+	"""
+	Given an iterable, return the items as a list. If the iterable contains
+	exactly one item, return that item. Correlary function to always_iterable.
 
-    >>> list_or_single(iter('abcd'))
-    ['a', 'b', 'c', 'd']
-    >>> list_or_single(['a'])
-    'a'
-    """
-    result = list(iterable)
-    if len(result) == 1:
-        result = result[0]
-    return result
+	>>> list_or_single(iter('abcd'))
+	['a', 'b', 'c', 'd']
+	>>> list_or_single(['a'])
+	'a'
+	"""
+	result = list(iterable)
+	if len(result) == 1:
+		result = result[0]
+	return result
+
+
+def self_product(iterable):
+	"""
+	Return the cross product of the iterable with itself.
+
+	>>> list(self_product([1, 2, 3]))
+	[(1, 1), (1, 2), ..., (3, 3)]
+	"""
+	return itertools.product(*itertools.tee(iterable))
